@@ -261,16 +261,47 @@ extern const struct s2n_security_policy security_policy_pq_tls_1_2_2024_10_09;
 
 extern const struct s2n_security_policy security_policy_cloudfront_upstream;
 extern const struct s2n_security_policy security_policy_cloudfront_upstream_tls10;
+extern const struct s2n_security_policy security_policy_cloudfront_upstream_tls11;
 extern const struct s2n_security_policy security_policy_cloudfront_upstream_tls12;
+/* CloudFront viewer facing */
 extern const struct s2n_security_policy security_policy_cloudfront_ssl_v_3;
 extern const struct s2n_security_policy security_policy_cloudfront_tls_1_0_2014;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_0_2014_sha256;
 extern const struct s2n_security_policy security_policy_cloudfront_tls_1_0_2016;
 extern const struct s2n_security_policy security_policy_cloudfront_tls_1_1_2016;
 extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2017;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2018_no_sha1;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2019_no_sha1;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2021_no_sha1;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2025;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_3_2025;
+/* CloudFront non-pq viewer facing */
+extern const struct s2n_security_policy security_policy_cloudfront_ssl_v_3_no_pq;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_0_2014_no_pq;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_0_2014_sha256_no_pq;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_0_2016_no_pq;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_1_2016_no_pq;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2017_no_pq;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2018_no_sha1_no_pq;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2019_no_sha1_no_pq;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2021_no_sha1_no_pq;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2025_no_pq;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_3_2025_no_pq;
+/* CloudFront undocumented policies for testing */
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_0_2014_pq_beta;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2021_no_sha1_pq_beta;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2018_beta;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2021_chacha20_boosted;
+/* CloudFront viewer facing legacy */
+extern const struct s2n_security_policy security_policy_cloudfront_ssl_v_3_legacy;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_0_2014_legacy;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_0_2016_legacy;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_1_2016_legacy;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2018_legacy;
+extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2019_legacy;
 extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2018;
 extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2019;
 extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2021;
-extern const struct s2n_security_policy security_policy_cloudfront_tls_1_2_2021_chacha20_boosted;
 
 extern const struct s2n_security_policy security_policy_kms_tls_1_0_2018_10;
 extern const struct s2n_security_policy security_policy_kms_tls_1_2_2023_06;
