@@ -86,6 +86,7 @@ extern const struct s2n_signature_preferences s2n_signature_preferences_20240501
 extern const struct s2n_signature_preferences s2n_signature_preferences_20230317;
 extern const struct s2n_signature_preferences s2n_signature_preferences_20140601;
 extern const struct s2n_signature_preferences s2n_signature_preferences_20200207;
+extern const struct s2n_signature_preferences s2n_signature_preferences_20200207_no_sha1;
 extern const struct s2n_signature_preferences s2n_signature_preferences_20201021;
 extern const struct s2n_signature_preferences s2n_signature_preferences_20210816;
 extern const struct s2n_signature_preferences s2n_signature_preferences_20240521;
@@ -97,6 +98,7 @@ extern const struct s2n_signature_preferences s2n_signature_preferences_default_
 extern const struct s2n_signature_preferences s2n_signature_preferences_null;
 extern const struct s2n_signature_preferences s2n_signature_preferences_test_all_fips;
 extern const struct s2n_signature_preferences s2n_signature_preferences_all;
+extern const struct s2n_signature_preferences s2n_signature_preferences_20250813;
 
 extern const struct s2n_signature_preferences s2n_certificate_signature_preferences_20251113;
 extern const struct s2n_signature_preferences s2n_certificate_signature_preferences_20250512;
